"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CoreQueue = exports.State = void 0;
var State;
(function (State) {
    State[State["running"] = 0] = "running";
    State[State["stopped"] = 1] = "stopped";
})(State = exports.State || (exports.State = {}));
class CoreQueue {
    // #endregion
    // #region constructor
    constructor(options) {
        // #region properties
        this.queue = new Array();
        this.tasksInProgress = 0;
        if (options) {
            this.options = options;
        }
        else {
            this.options = {
                maxTasks: 10000,
                maxConcurrency: 0,
                autostart: true,
            };
        }
        if (this.options.autostart) {
            this.queueState = State.running;
        }
    }
    // #endregion
    // #region getters
    get size() {
        return this.queue.length + this.tasksInProgress;
    }
    get pendingSize() {
        return this.queue.length;
    }
    get inProgressSize() {
        return this.tasksInProgress;
    }
    get isEmpty() {
        if (this.pendingSize === 0) {
            return true;
        }
        return false;
    }
    get isFull() {
        if (this.size === this.options.maxTasks) {
            return true;
        }
        return false;
    }
    get peek() {
        if (this.isEmpty) {
            return null;
        }
        return this.queue[0];
    }
    // #endregion
    // #region methods
    start() {
        this.queueState = State.running;
        this.dequeue();
    }
    stop() {
        this.queueState = State.stopped;
    }
    clear() {
        this.queue.length = 0;
    }
    async done() {
        await this.internalWatcher;
    }
    enqueue(func) {
        if (this.isFull) {
            throw new Error("Max number of task reached!");
        }
        return new Promise((resolve, reject) => {
            const task = {
                func,
                resolve,
                reject
            };
            this.queue.push(task);
            if (!this.internalWatcher) {
                this.internalWatcher = new Promise((r) => { this.internalWatcherResolve = r; });
            }
            this.dequeue();
        });
    }
    async dequeue() {
        if (this.queueState !== State.running) {
            return;
        }
        if (this.isEmpty) {
            return;
        }
        if (this.tasksInProgress >= this.options.maxConcurrency) {
            return;
        }
        this.tasksInProgress++;
        const task = this.queue.shift();
        try {
            const result = await task.func();
            task.resolve(result);
        }
        catch (error) {
            task.reject(error);
        }
        finally {
            this.tasksInProgress--;
            if (this.size === 0) {
                this.internalWatcherResolve();
                this.internalWatcher = null;
            }
            this.dequeue();
        }
    }
}
exports.CoreQueue = CoreQueue;
